%% BYOM, byom_guts_pred_lpx.m
%
% * Author: Tjalling Jager
% * Date: November 2021
% * Web support: <http://www.debtox.info/byom.html>
% * Back to index <walkthrough_guts.html>
%
% BYOM is a General framework for simulating model systems. The files in
% this directory use an analytical solution only, and therefore
% <derivatives.html derivatives.m> will be missing.
%
% *The model:* fitting survival data with the
% <http://www.debtox.info/about_guts.html GUTS> special cases based on the
% reduced model (TK and damage dynamics lumped): SD, IT and mixed (or GUTS
% proper). 
%
% *This script:* make predictions for an exposure profile, using a
% previously saved MAT file (generated by automatic_runs). This script
% makes use of the Matlab GUI element for selecting files from disk.
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(0) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% No data, just predictions

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,fname_prof,par] = select_pred([2 1 1]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters, as well as all settings used to generate the MAT
% file (in glo) apart from the exposure profile (which we'll redefine
% anyway). This only works when the MAT file was generated by BYOM v.6.0
% BETA 7 or newer as older versions won't save glo. 

% The parameter structure <par> is loaded from the selected MAT file. Even
% though it does not need to be used to call other functions, it must be
% specified for the code to run error free.

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios. All identifiers and initial values, as part of
% the matrix X0mat, are also loaded by select_pred already. Feel free to
% overwrite them with other values if needed.
% 
% NOTE: the glo.locD etc. have all been loaded from the MAT file by select_pred.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by
% select_pred already. Feel free to overwrite them with other values.

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% used, based on the data set

% All plot labels have been loaded from file using select_pred.
% Vector glo.t is already defined in select_pred from the selected profile.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculate LCx versus time
% Here, the LCx (by default the LC50) is calculated at several time points.
% LCx values are also printed on screen. If a sample from parameter space
% is available (e.g., from the slice sampler or the likelihood region), it
% can be used to calculate confidence bounds. 
% 
% Options for LCx (with confidence bounds) can be set using opt_lcx_lim
% (see prelim_checks). Note that opt_conf.type=-1 skips CIs.

opt_conf.type    = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
% Note: using the type that relates to the MAT file you selected!

opt_conf.lim_set = 2; % for lik-region sample: use limited set of n_lim points (1) or outer hull (2) to create CIs
opt_lcx_lim.Feff = 0.50; % effect level (>0 en <1), x/100 in LCx/LPx

Tend = [1:8]; % times at which to calculate LCx, relative to control
calc_lcx_lim_guts_red([],Tend,opt_lcx_lim,opt_conf); % calculates LCx values, CI requires that there is a mat file with sample

% % This is the slower general method
% opt_ecx.Feff      = [0.10 0.50]; % effect levels (>0 en <1), x/100 in ECx
% opt_ecx.notitle   = 1; % set to 1 to suppress titles above ECx plots
% calc_ecx([],Tend,opt_ecx,opt_conf); % general method for ECx values

%% Calculate LPx
% We can use the function calc_lpx_lim to calculate an exposure
% multiplication factor: with which factor do we need to multiply an
% exposure scenario to obtain x% effect at the end of the scenario? Here,
% done for 10% effect at the end of the exposure scenario. Note that
% opt_conf.type=-1 skips CIs.
%
% This is the fast method, which is closely linked to this GUTS model
% (reduced model, standard directory). 

opt_conf.type    = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
% Note: using the type that relates to the MAT file you selected!

opt_conf.lim_set    = 2; % for lik-region sample: use limited set of n_lim points (1) or outer hull (2) to create CIs
opt_lcx_lim.Feff    = 0.10; % effect level (>0 en <1), x/100 in LCx/LPx
opt_lcx_lim.notitle = 0; % set to 1 to suppress titles above ECx plots

Tend    = []; % time at which to calculate LPx (empty is end of profile)
LPx_mon = calc_lpx_lim_guts_red([],Tend,fname_prof,opt_lcx_lim,opt_conf);

% % This is the slower general method
% opt_ecx.Feff      = [0.10]; % effect levels (>0 en <1), x/100 in ECx
% opt_ecx.notitle   = 1; % set to 1 to suppress titles above ECx plots
% calc_epx([],fname_prof,[],opt_ecx,opt_conf,opt_tktd); % general method for EPx values
